@@ -1,6 +1,6 @@
 import os
 import pytest
-from mp_api import MPRester
+from mp_api.client import MPRester
 
 # -- Rester name data for generic tests
 
@@ -30,10 +30,12 @@ ignore_generic = [
     "_user_settings",
     "_general_store",
     "tasks",
+    "bonds",
     "xas",
     "elasticity",
+    "fermi",
     "alloys",
-    "summary"
+    "summary",
 ]  # temp
 
 
